% EXAMPLE_2D Demonstrates translation-invariant hereditary Haar 
% deblurring of a Poisson image

% generate true image
load confocalPhantom;
f = lam*4;

% generate blur
h1 = exp(-.1*abs((0:20)-10).^2);
h = h1;
lh = length(h);
h = h/sum(h);
h = [h zeros(1,length(lam)-length(h))];
h = fftshift(h);
h = h'*h;
fh = fft2(h);
h1 = h1/sum(h1);
pp = h1'*h1;

% generate blurry image
%mu = real(ifft2(fft2(f).*fh));
mu = conv2b(f,pp);

% generate noisy observations
y = randp(mu);
fy = fft2(y);

% display true and noisy images
figure(23);
imagesc([f,mu,y],[min(f(:)) max(f(:))*1.3]); axis image;colormap gray

% initialize to flat image
f_hat = ones(size(f))*mean(y(:));
estError = [];
percentChange = [];

% make of moive of the results
% mov = avifile('TomographyMovie.avi','fps',1);

for i = 1:4000
  prev_f_hat = f_hat;
  
  % E-step
  %z_{n,m} = (y_n lam_m p_{n,m})/(\sum_{l=0}^{2^J-1} lam_l p_{n,l})
  z = f_hat.*conv2b(y./(conv2b(f_hat,pp)+realmin),pp);
  
  % M-step
  f_hat = haarTIApprox(z,log(sum(z(:)))/20);
  
  estError(i) = mean((f(:)-f_hat(:)).^2)/mean((f(:).^2));
  percentChange(i) = mean((prev_f_hat(:)-f_hat(:)).^2)/mean((prev_f_hat(:).^2));
  fprintf(1,'\niter = %4d\testError = %4d\tpercentChange = %4d\n',...
    i,estError(i),percentChange(i));

  figure(24);
  imagesc([f y f_hat],[min(f(:)) max(f(:))*1.3]); axis image; colormap gray
  title('True Image -- Reconstructed Image');
  h=text(1,3,['Iter ' int2str(i) ', MSE = ' num2str(estError(i))]);
  set(h,'Color',[1 1 1])
  set(h,'FontSize',16)
  %  mov = addframe(mov,getframe(gca));
  
  figure(25)
  subplot(211); semilogy(percentChange,'kx-'); 
  title('percent change from last iteration'); xlabel('iteration');
  subplot(212); semilogy(estError,'kx-');
  title('MSE for current estimate'); xlabel('iteration');
  pause(.01)
      
  if (percentChange(i) < 10^(-6))
    numIter = i;
    break;
  end
end

% mov = close(mov);

figure; imagesc(f,[min(f(:)) max(f(:))*1.3]);
axis image; colormap gray; axis off
print -deps confocalTrue.eps

figure; imagesc(y,[min(f(:)) max(f(:))*1.3]);
axis image; colormap gray; axis off
print -deps confocalObs.eps

figure; imagesc(f_hat,[min(f(:)) max(f(:))*1.3]);
axis image; colormap gray; axis off
print -deps confocalEst.eps
