% EXAMPLE_2D Demonstrates translation-invariant hereditary Haar tomographic 
% reconstruction of a Poisson image

% generate true image
f = shepplogan'*1000;

% load transition matrix
load pmat;

% generate true sinogram
sino = pmat*f(:);

% generate noisy observations
y = randp(sino);

% display true and noisy sinograms
figure(23);
imagesc([reshape(sino,64,64),reshape(y,64,64)]); axis image;colormap gray

% initialize to flat image
f_hat = ones(size(f))*mean(y(:));
estError = [];
percentChange = [];

% make of moive of the results
% mov = avifile('TomographyMovie.avi','fps',1);

for i = 1:4000
  prev_f_hat = f_hat;
  
  % E-step
  %z_{n,m} = (y_n lam_m p_{n,m})/(\sum_{l=0}^{2^J-1} lam_l p_{n,l})
  ll = y./(pmat*f_hat(:)+realmin);
  x = (ll*(f_hat(:))'.*pmat)'*ones(length(ll),1);
  x = reshape(x,64,64);

  % M-step
  f_hat = haarTIApprox(x,log(sum(x(:)))/20);
  
  estError(i) = mean((f(:)-f_hat(:)).^2)/mean((f(:).^2));
  percentChange(i) = mean((prev_f_hat(:)-f_hat(:)).^2)/mean((prev_f_hat(:).^2));
  fprintf(1,'\niter = %4d\testError = %4d\tpercentChange = %4d\n',...
    i,estError(i),percentChange(i));

  figure(24);
  imagesc([f f_hat],[min(f(:)) max(f(:))]); axis image; colormap gray
  title('True Image -- Reconstructed Image');
  h=text(1,3,['Iter ' int2str(i) ', MSE = ' num2str(estError(i))]);
  set(h,'Color',[1 1 1])
  set(h,'FontSize',16)
  %  mov = addframe(mov,getframe(gca));
  
  figure(25)
  subplot(211); semilogy(percentChange,'kx-'); 
  title('percent change from last iteration'); xlabel('iteration');
  subplot(212); semilogy(estError,'kx-');
  title('MSE for current estimate'); xlabel('iteration');
  pause(.01)
      
  if (percentChange(i) < 10^(-5))
    numIter = i;
    break;
  end
end

% mov = close(mov);