/*
 y = haarTIApprox(x,pen);
 */

#include <math.h>
#include <stdio.h>
#include "mex.h"
#include "matrix.h"
#include "mathHelper.c"
#include "pruneTree1D.c"
#include "pruneTree2D.c"
#include "pruneTree3D.c"
#include "pruneTree3D_BigData.c"

void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
  double *x, *y;
  double **x2, **y2;
  double ***x3, ***y3;
  int im1, im2, im3, m1,m2,m3;
  int i, j, L, nDim, nElts;
  const int *dims;
  double mtest, ntest, pen, sum = 0;
  char *noiseType;

  /* check for correct # of input variables */
  if (nrhs>3){
    mexErrMsgTxt("There are at most 3 input parameters allowed!");
    return;
  }
  if (nrhs<1){
    mexErrMsgTxt("There is at least 1 input parameter required!");
    return;
  }
  x = mxGetPr(prhs[0]);
  nDim = mxGetNumberOfDimensions(prhs[0]);
  dims = mxGetDimensions(prhs[0]);
  nElts = mxGetNumberOfElements(prhs[0]);
  if (nDim > 3) {
    mexErrMsgTxt("The input array cannot have more than three dimensions.");
    return;
  }

  /* set noiseType */
  if (nrhs < 3) {
    noiseType = "Poisson";
  }
  else {
    if (mxIsChar(prhs[2]) != 1)
      mexErrMsgTxt("noiseType must be a string");
    i = (mxGetM(prhs[2])*mxGetN(prhs[2])) + 1;
    noiseType = mxCalloc(i,sizeof(char));
    mxGetString(prhs[2],noiseType,i);
  }
  if ((strcmp(noiseType,"Poisson") != 0) && (strcmp(noiseType,"Gaussian")!=0)) {
    mexErrMsgTxt("noiseType must be either 'Poisson' or 'Gaussian'.");
  }
  if (strcmp(noiseType,"Gaussian")==0) {
    mexErrMsgTxt("This feature is not yet enabled.");
  }

  /* set penalty
    = log(sum(x))/2 for Poisson noise
   = log(prod(size(x)))/2 for Gaussian noise
   */
  if ((nrhs < 2)||(mxGetN(prhs[1])==0)) {
    if (strcmp(noiseType,"Poisson")==0) {
      for (i = 0; i < nElts; i++) {
        sum += *(x+i);
      }
      if (sum < 0) {
        mexErrMsgTxt("Poisson data cannot be negative.");
      }
      pen = (double) log(sum)/2.0;
    }
    else if (strcmp(noiseType,"Gaussian")==0) {
      pen = (double) log(nElts)/2.0;
    }
    else {
      mexErrMsgTxt("noiseType must be either 'Poisson' or 'Gaussian'.");
    }
  }
  else {
    pen = (double) *mxGetPr(prhs[1]);
  }
  if (pen < 0) {
    mexErrMsgTxt("The penalty must be positive.");
  }

  /* Check the dimensions of input */
  for (i = 0; i < nDim; i++) {
    mtest = (double) log((mxGetDimensions(prhs[0]))[i])/log(2.0);
    if (!isint(mtest)) {
      mexErrMsgTxt("Each matrix dimension must be of size m*2^(L)");
    }
  }
  plhs[0] = mxCreateNumericArray(nDim,dims,mxDOUBLE_CLASS,mxREAL);
  y = mxGetPr(plhs[0]);
  if (nDim==1) {
    pruneTree1D(x, (mxGetDimensions(prhs[0]))[0], pen, y, noiseType);
  }
  else if (nDim==2) {
    m1 = (int)(mxGetDimensions(prhs[0]))[0];
    m2 = (int)(mxGetDimensions(prhs[0]))[1];
    if (m1==1) {
      pruneTree1D(x, m2, pen, y, noiseType);
    }
    else if (m2==1) {
      pruneTree1D(x, m1, pen, y, noiseType);
    }
    else {
    x2 = (double **)mxCalloc(m1,sizeof(double*));
    y2 = (double **)mxCalloc(m1,sizeof(double*));
    for (im1 = 0; im1 < m1; im1++) {
      x2[im1] = (double *)mxCalloc(m2,sizeof(double));
      y2[im1] = (double *)mxCalloc(m2,sizeof(double));
      for (im2 = 0; im2 < m2; im2++) {
        x2[im1][im2] = x[im1+im2*m1];
      }
    }
    pruneTree2D(x2, m1, m2, pen, y2, noiseType);
    for (im1 = 0; im1 < m1; im1++) {
      for (im2 = 0; im2 < m2; im2++) {
        y[im1+im2*m1] = y2[im1][im2];
      }
    }
    }
  }
  else {
    m1 = (int)(mxGetDimensions(prhs[0]))[0];
    m2 = (int)(mxGetDimensions(prhs[0]))[1];
    m3 = (int)(mxGetDimensions(prhs[0]))[2];
    if ((m1==1)&&(m2==1)) {
      pruneTree1D(x, m3, pen, y, noiseType);
    }
    else if ((m1==1)&&(m3==1)) {
      pruneTree1D(x, m2, pen, y, noiseType);
    }
    else if ((m2==1)&&(m3==1)) {
      pruneTree1D(x, m1, pen, y, noiseType);
    }
    else {
    x3 = (double ***)mxCalloc(m1,sizeof(double**));
    y3 = (double ***)mxCalloc(m1,sizeof(double**));
    for (im1 = 0; im1 < m1; im1++) {
      x3[im1] = (double **)mxCalloc(m2,sizeof(double*));
      y3[im1] = (double **)mxCalloc(m2,sizeof(double*));
      for (im2 = 0; im2 < m2; im2++) {
        x3[im1][im2] = (double *)mxCalloc(m3,sizeof(double));
        y3[im1][im2] = (double *)mxCalloc(m3,sizeof(double));
        for (im3 = 0; im3 < m3; im3++) {
          x3[im1][im2][im3] = x[im1+im2*m1+im3*m1*m2];
          y3[im1][im2][im3] = 0;
        }
      }
    }
    if (0) 
      pruneTree3D(x3, m1, m2, m3, pen, y3, noiseType); 
    else
      pruneTree3D_BigData(x3, m1, m2, m3, pen, y3, noiseType);

    for (im1 = 0; im1 < m1; im1++) {
      for (im2 = 0; im2 < m2; im2++) {
        for (im3 = 0; im3 < m3; im3++) {
          y[im1+im2*m1+im3*m1*m2] = y3[im1][im2][im3];
        }
      }
    }
    }
  }
}
