function P = PHANTOM3D(N,K);
%   P = PHANTOM(E,N) generates a user-defined phantom, where each row
%   of the matrix E specifies an ellipse in the image.  E has six columns,
%   with each column containing a different parameter for the ellipses:
%   
%     Column 1:  A    the additive intensity value of the ellipse
%     Column 2:  a    the length of the horizontal semi-axis of the ellipse 
%     Column 3:  b    the length of the vertical semi-axis of the ellipse
%     Column 4:  x0   the x-coordinate of the center of the ellipse
%     Column 5:  y0   the y-coordinate of the center of the ellipse
%     Column 6:  phi  the angle (in degrees) between the horizontal semi-axis 
%                     of the ellipse and the x-axis of the image        
%
%   For purposes of generating the phantom, the domains for the x- and 
%   y-axes span [-1,1].  Columns 2 through 5 must be specified in terms
%   of this range.
%
%   [P,E] = PHANTOM(...) returns the matrix E used to generate the phantom.
%
      
%
%   This head phantom is the same as the Shepp-Logan except 
%   the intensities are changed to yield higher contrast in
%   the image.  Taken from Toft, 199-200.
%      
%         A    a     b    x0    y0    phi
%        ---------------------------------
toft = [  
  1   .69   .92    0     0     0   
  -.8  .6624 .8740   0  -.0184   0
  -.2  .1100 .3100  .22    0    -18
  -.2  .1600 .4100 -.22    0     18
  .1  .2100 .2500   0    .35    0
  .1  .0460 .0460   0    .1     0
  .1  .0460 .0460   0   -.1     0
  .1  .0460 .0230 -.08  -.605   0 
  .1  .0230 .0230   0   -.606   0
  .1  .0230 .0460  .06  -.605   0   ];

if nargin==1
  K = 16;
end
P = zeros(N,N,K);
toftk = toft;
Pk(:,:,1) = phantom(toft,N);
P(:,:,1) = Pk;
%figure(1);imagesc(Pk(:,:,1));axis image;colormap gray
c1 = 1.1^(1/K);
c2 = 0.9^(1/K);
for k = 2:K;
  toftk(:,2) = toftk(:,2)*c1;
  toftk(:,3) = toftk(:,3)*c2;
  toftk(:,4) = toftk(:,4)*c1;
  toftk(:,5) = toftk(:,5)*c2;
  toftk(:,6) = toftk(:,6)*c1;
  Pk = phantom(toftk,N);
  P(:,:,k) = Pk;
%  figure(k);imagesc(Pk);axis image;colormap gray;
end
