void pruneTree1D(double *x, int n, double pen, double *y, char *noiseType)
{
  /* int L,in,iL,j,k,iL2, dyadLen; */
  int L,in,iL,k,dyadLen;
  double pMerge,pSplit;
  double *xScaling, *xScalingPrev, *xDifference, *optProb,
    *mergeProb, *waveletScale, *splitDecision;
  double xD1, xD2;

  L = (int) floor(log(n)/log((double)2));
  xScaling = (double *)mxCalloc(n,sizeof(double));
  xScalingPrev = (double *)mxCalloc(n,sizeof(double));
  waveletScale = (double *)mxCalloc(n*L,sizeof(double));
  xDifference = (double *)mxCalloc(n*L,sizeof(double));
  splitDecision = (double *)mxCalloc(n*L,sizeof(double));
  optProb = (double *)mxCalloc(n*(L+1),sizeof(double));
  mergeProb = (double *)mxCalloc(n*(L+1),sizeof(double));
  /* initialize variables */
  for (in = 0; in < n; in++){
    *(xScalingPrev +in) = *(x+in);
    *(y+in) = *(x+in);
    /* *(optProb + in) = *(x+in)*log(*(x+in))-*(x+in)-pen; */
    *(optProb+in) = -logLike1D(x,x,1,in,noiseType)-pen;
    if (strcmp(noiseType,"Poisson")==0) {
      *(mergeProb + in) = 0;
    }
    else if (strcmp(noiseType,"Gaussian")==0) {
      *(mergeProb+in) = -pow(*(x+in),2.0);
    }
  }
  for (iL=0;iL<L;iL++) { /* iL is the scale */
    dyadLen = (int)pow(2.0,iL+1); /* length of the current dyadic interval */
    /* iterate through each scaling coefficient from the previous scale */
    for (in = 0; in < n; in++){
      k = (int)mod(in+dyadLen/2,n);
      /* calculate current scaling coefficient */
      *(xScaling+in) = (*(xScalingPrev+in)+*(xScalingPrev+k))/2.0;
      /* log probability of merging */
      if (strcmp(noiseType,"Poisson")==0) {
        pMerge = *(mergeProb+in+iL*n) + *(mergeProb+k+iL*n) +
        -*(xScaling+in)*dyadLen*(1.0-log(*(xScaling+in)*dyadLen)-log(0.5))
        - pen;
      }
      else if (strcmp(noiseType,"Gaussian")==0) {
        pMerge = *(mergeProb+in+iL*n) + *(mergeProb+k+iL*n) + pow(*(xScaling+in),2.0)*dyadLen
        - pen;
      }
      /* log probability of splitting */
      pSplit = *(optProb+in+iL*n) + *(optProb+k+iL*n);
      /* terms of merge log probability needed to calculated pMerge at next scale */
      if (strcmp(noiseType,"Poisson")==0) {
        *(mergeProb+in+(iL+1)*n) = *(mergeProb+in+iL*n) + *(mergeProb+k+iL*n) +
        *(xScaling+in)*dyadLen*log(0.5);
      }
      else if (strcmp(noiseType,"Gaussian")==0) {
        *(mergeProb+in+(iL+1)*n) = *(mergeProb+in+iL*n) + *(mergeProb+k+iL*n);
      }
      /* "wavelet" coefficients */
      *(xDifference+in+iL*n) = *(xScalingPrev+in)-*(xScaling+in);
      /* decide whether to split or merge, save decision and associated
        log probability */
      if (pMerge >= pSplit) {
        *(splitDecision+in+iL*n) = 0;
        *(optProb+in+(iL+1)*n) = pMerge;
      }
      else {
        *(splitDecision+in+iL*n) = 1.0;
        *(optProb+in+(iL+1)*n) = pSplit;
      }
      /* initialize waveletScale = 1; means the wavelets are not thresholded */
      *(waveletScale+in+iL*n) = 1.0;
    }
    /* set up scaling coefficients for next scale */
    for (in = 0; in < n; in++) {
      *(xScalingPrev+in) = *(xScaling+in);
    }
  }
  /* initial estimate is coarse scale scaling coefficients */
  for (in = 0; in < n; in++) {
    *(y+in) = *(xScaling+in);
  }
  for (iL=L-1; iL>=0; iL--) {
    dyadLen = (int)pow(2.0,iL+1);
    for (in = 0; in < n; in++) {
      k = (int)mod(in+dyadLen/2,n);
      /* if the split decision is zero, then the associated wavelet should
      be set to zero. */
      if ((*(splitDecision+in+iL*n)==0)) {
        *(waveletScale+in+iL*n) = 0;
      }
      if (iL > 0) {
        /* threshold wavelet coefficient based on ancestors' split/merge
        decisions */
        *(waveletScale+in+(iL-1)*n) = *(waveletScale+in+(iL-1)*n)
        - (0.5*(1.0-*(waveletScale+in+iL*n)));
        *(waveletScale+ k+(iL-1)*n) = *(waveletScale+ k+(iL-1)*n)
          - (0.5*(1.0-*(waveletScale+in+iL*n)));
      }
    }
    /*	 construct estimate based on wavelet coefficients and thresholds */
    for (in = 0; in < n; in++) {
      k = (int)mod(in-dyadLen/2.0,n);
      xD1 = *(xDifference+in+iL*n)*(*(waveletScale+in+iL*n));
      xD2 = *(xDifference+ k+iL*n)*(*(waveletScale+ k+iL*n));
      *(xScaling+in) = (*(y+in)+xD1 + *(y+k)-xD2)/2.0;
    }
    for (in = 0; in < n; in++) {
      *(y+in) = *(xScaling+in);
    }
  }
}	
