void pruneTree2D(double **x, int m, int n, double pen, double **y, char *noiseType)
{
  int L,in,im,iL,j,kn,km,iL2, dyadLen;
  double pMerge,pSplit,pWedge;
  double **xScaling, **xScalingPrev;
  double ***optProb, ***mergeProb, ***waveletScale, ***splitDecision,
    ***wavelet_n, ***wavelet_m, ***wavelet_k;
  double xD1, xD2, xD3, xD4;

  L = (int) floor(log(n)/log((double)2));
  
  xScaling = (double **)mxCalloc(m,sizeof(double*));
  xScalingPrev = (double **)mxCalloc(m,sizeof(double*));

  waveletScale = (double ***)mxCalloc(m,sizeof(double**));
  wavelet_n = (double ***)mxCalloc(m,sizeof(double**));
  wavelet_m = (double ***)mxCalloc(m,sizeof(double**));
  wavelet_k = (double ***)mxCalloc(m,sizeof(double**));
  splitDecision = (double ***)mxCalloc(m,sizeof(double**));
  optProb = (double ***)mxCalloc(m,sizeof(double**));
  mergeProb = (double ***)mxCalloc(m,sizeof(double**));

  for (im = 0; im < m; im++) {
    xScaling[im] = (double *)mxCalloc(n,sizeof(double));
    xScalingPrev[im] = (double *)mxCalloc(n,sizeof(double));

    waveletScale[im] = (double **)mxCalloc(n,sizeof(double*));
    wavelet_n[im] = (double **)mxCalloc(n,sizeof(double*));
    wavelet_m[im] = (double **)mxCalloc(n,sizeof(double*));
    wavelet_k[im] = (double **)mxCalloc(n,sizeof(double*));
    splitDecision[im] = (double **)mxCalloc(n,sizeof(double*));
    optProb[im] = (double **)mxCalloc(n,sizeof(double*));
    mergeProb[im] = (double **)mxCalloc(n,sizeof(double*));

    for (in = 0; in <n; in++) {
      waveletScale[im][in] = (double *)mxCalloc(L,sizeof(double));
      wavelet_n[im][in] = (double *)mxCalloc(L,sizeof(double));
      wavelet_m[im][in] = (double *)mxCalloc(L,sizeof(double));
      wavelet_k[im][in] = (double *)mxCalloc(L,sizeof(double));
      splitDecision[im][in] = (double *)mxCalloc(L,sizeof(double));
      optProb[im][in] = (double *)mxCalloc(L+1,sizeof(double));
      mergeProb[im][in] = (double *)mxCalloc(L+1,sizeof(double));
    }
  }
  /* initialize variables */
  for (im = 0; im < m; im++) {
    for (in = 0; in < n; in++){
      xScalingPrev[im][in] = x[im][in];
      y[im][in] = x[im][in];
      optProb[im][in][0] = logLike2D(x,x,1,1,im,in,noiseType)-pen;
      if (strcmp(noiseType,"Poisson")==0) {
        mergeProb[im][in][0] = 0;
      }
      else if (strcmp(noiseType,"Gaussian")==0) {
        mergeProb[im][in][0] = -pow(x[im][in],2.0);
      }
    }
  }


  for (iL=0;iL<L;iL++) {
    dyadLen = (int)pow(2.0,iL+1);
    for (im = 0; im < m; im++) {
      for (in = 0; in < n; in++){
        km = (int)mod(im+dyadLen/2,m);
        kn = (int)mod(in+dyadLen/2,n);

        /* calculate current scaling coefficient */
        xScaling[im][in] = (xScalingPrev[im][in]+xScalingPrev[im][kn]+
                            xScalingPrev[km][in]+xScalingPrev[km][kn])/4.0;

        /* log probability of merging */
        if (strcmp(noiseType,"Poisson")==0) {
          pMerge = (mergeProb[im][in][iL] + mergeProb[im][kn][iL] +
                    mergeProb[km][in][iL] + mergeProb[km][kn][iL] -
                    xScaling[im][in]*dyadLen*dyadLen*
                    (1.0-log(xScaling[im][in]*dyadLen*dyadLen)-log(0.25))
                    - pen);
          }
        else if (strcmp(noiseType,"Gaussian")==0) {
          pMerge = (mergeProb[im][in][iL] + mergeProb[im][kn][iL] +
                    mergeProb[km][in][iL] + mergeProb[km][kn][iL]
                    + pow(xScaling[im][in],2.0)*dyadLen*dyadLen
                    - pen);
        }

        /* log probability of splitting */
        pSplit = (optProb[im][in][iL] + optProb[im][kn][iL] +
                  optProb[km][in][iL] + optProb[km][kn][iL]);

        /* terms of merge log probability needed to calculated pMerge at next scale */
        if (strcmp(noiseType,"Poisson")==0) {
          mergeProb[im][in][iL+1] = (mergeProb[im][in][iL] + mergeProb[im][kn][iL] +
                                      mergeProb[km][in][iL] + mergeProb[km][kn][iL] +
                                      xScaling[im][in]*dyadLen*dyadLen*log(0.25));
        }
        else if (strcmp(noiseType,"Gaussian")==0) {
          mergeProb[im][in][iL+1] = (mergeProb[im][in][iL] + mergeProb[im][kn][iL] +
                                      mergeProb[km][in][iL] + mergeProb[km][kn][iL]);
        }

        /* "wavelet" coefficients */
        wavelet_n[im][in][iL] = ((xScalingPrev[im][in]+xScalingPrev[km][in])/2.0
                                 -xScaling[im][in]);
        wavelet_m[im][in][iL] = ((xScalingPrev[im][in]+xScalingPrev[im][kn])/2.0
                                 -xScaling[im][in]);
        wavelet_k[im][in][iL] = ((xScalingPrev[im][in]+xScalingPrev[km][kn])/2.0
                                 -xScaling[im][in]);

        
        /* decide whether to split or merge, save decision and associated
           log probability */
        if (pMerge >= pSplit) {
          splitDecision[im][in][iL] = 0;
          optProb[im][in][iL+1] = pMerge;
        }
        else {
          splitDecision[im][in][iL] = 1.0;
          optProb[im][in][iL+1] = pSplit;            
        }
        waveletScale[im][in][iL] = 1.0;
      }
    }


    /* set up scaling coefficients for next scale */
    for (im = 0; im < m; im++) {
      for (in = 0; in < n; in++) {
        xScalingPrev[im][in] = xScaling[im][in];
      }
    }
  }

  /* initial estimate is coarse scale scaling coefficients */
  for (im = 0; im < m; im++) {
    for (in = 0; in < n; in++) {
      y[im][in] = xScaling[im][in];
    }
  }

  for (iL=L-1; iL>=0; iL--) {
    dyadLen = (int)pow(2.0,iL+1);

    for (in = 0; in < n; in++) {
      for (im = 0; im < m; im++) {
        kn = (int)mod(in+dyadLen/2,n);
        km = (int)mod(im+dyadLen/2,m);
        /* if the split decision is zero, then the associated wavelet should
           be set to zero. */
        if (splitDecision[im][in][iL]==0) {
          waveletScale[im][in][iL] = 0;
        }
        if (iL > 0) {
          /* threshold wavelet coefficient based on ancestors' split/merge
             decisions */
          waveletScale[im][in][iL-1] = waveletScale[im][in][iL-1]
            - (0.25*(1.0-waveletScale[im][in][iL]));
          waveletScale[im][kn][iL-1] = waveletScale[im][kn][iL-1]
            - (0.25*(1.0-waveletScale[im][in][iL]));
          waveletScale[km][in][iL-1] = waveletScale[km][in][iL-1]
            - (0.25*(1.0-waveletScale[im][in][iL]));
          waveletScale[km][kn][iL-1] = waveletScale[km][kn][iL-1]
            - (0.25*(1.0-waveletScale[im][in][iL]));
        }
      }
    }

    /* construct estimate based on wavelet coefficients and thresholds */
    for (in = 0; in < n; in++) {
      for (im = 0; im < m; im++) {
        kn = (int)mod(in-dyadLen/2,n);
        km = (int)mod(im-dyadLen/2,m);

        xD1 = (wavelet_n[im][in][iL]+wavelet_m[im][in][iL]+
               wavelet_k[im][in][iL])*(waveletScale[im][in][iL]);

        xD2 = (wavelet_n[im][kn][iL]-wavelet_m[im][kn][iL]+
               wavelet_k[im][kn][iL])*(waveletScale[im][kn][iL]);

        xD3 = (-wavelet_n[km][in][iL]+wavelet_m[km][in][iL]+
               wavelet_k[km][in][iL])*(waveletScale[km][in][iL]);

        xD4 = (-wavelet_n[km][kn][iL]-wavelet_m[km][kn][iL]+
               wavelet_k[km][kn][iL])*(waveletScale[km][kn][iL]);

        xScaling[im][in] = (y[im][in]+xD1 + y[im][kn]-xD2 +
                            y[km][in]-xD3 + y[km][kn]+xD4)/4.0;
      }
    }
    for (im = 0; im < m; im++) {
      for (in = 0; in < n; in++) {
        y[im][in] = xScaling[im][in];
      }
    }
  }
}
